`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/05/13 16:30:11
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////
`include "global.vh"

module topmodule(
    D_ssd,
    ssd_ctl,
    clk,
    rst,
    PS2_DATA,
    PS2_CLK
    );

    output [7:0] D_ssd;
    output [3:0] ssd_ctl;
    input clk;
    input rst;
    inout PS2_CLK;
    inout PS2_DATA;
    
    wire clk_1Hz;
    wire clk_100Hz;
    wire [1:0] clk_ssd_ctl;

    wire [511:0] key_down;
    wire [8:0] last_change;
    wire key_valid;
    wire [3:0] decode_result;
    
    wire valid_signal;

    wire [3:0] digit_0, digit_1;
    wire [3:0] sum, carry_out;

    wire curr_state;

    wire [3:0] ssd_in;

    assign valid_signal = key_down[last_change];

    clock_generator U0(
    .clk_1Hz(clk_1Hz), //divided clock output
    .clk_100Hz(clk_100Hz),
    .clk_ctl(clk_ssd_ctl), //clock for 7-segment
    .clk(clk),    //clock input
    .rst(rst)   //low-active reset
    );

    KeyboardDecoder U1(
	.key_down(key_down),
	.last_change(last_change),
	.key_valid(key_valid),
	.PS2_DATA(PS2_DATA),
	.PS2_CLK(PS2_CLK),
	.rst(rst),
	.clk(clk)
    );
    
    key_decoder U2(
    .result(decode_result), 
    .select_in(last_change),
    .clk(clk),
    .rst(rst)
    );

    fsm U3(
    .last_change(last_change),
    .valid(valid_signal),
    .curr_state(curr_state),
    .rst(rst)
);
    value_temp_reg U4(
    .digit_0(digit_0),
    .digit_1(digit_1),
    .curr_state(curr_state),
    .valid(valid_signal),
    .decoded_input(decode_result),
    .rst(rst)
);
    dec_adder U5(
    .cout(carry_out), 
    .sum(sum), 
    .num_0(digit_0), 
    .num_1(digit_1),
    .clk(clk),
    .rst(rst)
    );

    scan_freq_ctl U6(
    .ssd_in(ssd_in),
    .ssd_ctl(ssd_ctl),
    .clk_ctl_en(clk_ssd_ctl),
    .in_3(digit_0),
    .in_2(digit_1),
    .in_1(carry_out),
    .in_0(sum)
);

    display U7(.D_ssd(D_ssd), .i(ssd_in));

endmodule
